@@ -5,27 +5,25 @@ Supported devices:
 - Corsair Commander Pro
 - Corsair Lighting Node Pro
 
+Important notes:
+- This device currently only has hardware control implemented but it also
+  supports a software control mode.
+- Software control will be enabled at a future time.
 
-NOTE:
-    This device currently only has hardware control implemented but it also supports a software control mode.
-    Software control will be enabled at a future time.
-
-
-Copyright (C) 2020–2021  Marshall Asch and contributors
+Copyright Marshall Asch and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
 """
 
 import itertools
 import logging
 import re
-
 from enum import Enum, unique
 
 from liquidctl.driver.usb import UsbHidDriver
-from liquidctl.keyval import RuntimeStorage
-from liquidctl.pmbus import compute_pec
-from liquidctl.util import clamp, fraction_of_byte, u16be_from, u16le_from, normalize_profile, check_unsafe
 from liquidctl.error import NotSupportedByDevice
+from liquidctl.keyval import RuntimeStorage
+from liquidctl.util import clamp, fraction_of_byte, u16be_from, u16le_from, \
+                           normalize_profile, check_unsafe, map_direction
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -42,6 +40,7 @@ _CMD_GET_FAN_MODES = 0x20
 _CMD_GET_FAN_RPM = 0x21
 _CMD_SET_FAN_DUTY = 0x23
 _CMD_SET_FAN_PROFILE = 0x25
+_CMD_SET_FAN_MODE = 0x28
 
 _CMD_RESET_LED_CHANNEL = 0x37
 _CMD_BEGIN_LED_EFFECT = 0x34
@@ -67,6 +66,7 @@ _PROFILE_LENGTH = 6
 _CRITICAL_TEMPERATURE = 60
 _CRITICAL_TEMPERATURE_HIGH = 100
 _MAX_FAN_RPM = 5000             # I have no idea if this is a good value or not
+_MAX_LEDS = 204
 
 _MODES = {
     'off': 0x04,            # this is a special case of fixed
@@ -83,6 +83,11 @@ _MODES = {
     'rainbow2': 0x0a,
 }
 
+_FAN_MODES = {
+    'off': _FAN_MODE_DISCONNECTED,
+    'dc': _FAN_MODE_DC,
+    'pwm': _FAN_MODE_PWM,
+}
 
 def _prepare_profile(original, critcalTempature):
     clamped = ((temp, clamp(duty, 0, _MAX_FAN_RPM)) for temp, duty in original)
@@ -99,28 +104,34 @@ def _quoted(*names):
     return ', '.join(map(repr, names))
 
 
-def _get_fan_mode_description(mode):
+def _fan_mode_desc(mode):
     """This will convert the fan mode value to a descriptive name.
     """
 
-    if mode == _FAN_MODE_DISCONNECTED:
-        return 'Auto/Disconnected'
-    elif mode == _FAN_MODE_DC:
+    if mode == _FAN_MODE_DC:
         return 'DC'
     elif mode == _FAN_MODE_PWM:
         return 'PWM'
     else:
-        return 'UNKNOWN'
+        if mode != _FAN_MODE_DISCONNECTED:
+            _LOGGER.warning('unknown fan mode: {mode:#04x}')
+        return None
 
 
 class CommanderPro(UsbHidDriver):
     """Corsair Commander Pro LED and fan hub"""
 
-    SUPPORTED_DEVICES = [
-        (0x1b1c, 0x0c10, None, 'Corsair Commander Pro (experimental)',
+    # support for hwmon: corsair-cpro, Linux 5.9
+
+    _MATCHES = [
+        (0x1b1c, 0x0c10, 'Corsair Commander Pro',
             {'fan_count': 6, 'temp_probs': 4, 'led_channels': 2}),
-        (0x1b1c, 0x0c0b, None, 'Corsair Lighting Node Pro (experimental)',
+        (0x1b1c, 0x0c0b, 'Corsair Lighting Node Pro',
             {'fan_count': 0, 'temp_probs': 0, 'led_channels': 2}),
+        (0x1b1c, 0x0c1a, 'Corsair Lighting Node Core',
+            {'fan_count': 0, 'temp_probs': 0, 'led_channels': 1}),
+        (0x1b1c, 0x1d00, 'Corsair Obsidian 1000D',
+            {'fan_count': 6, 'temp_probs': 4, 'led_channels': 2}),
     ]
 
     def __init__(self, device, description, fan_count, temp_probs, led_channels, **kwargs):
@@ -129,7 +140,10 @@ class CommanderPro(UsbHidDriver):
         # the following fields are only initialized in connect()
         self._data = None
         self._fan_names = [f'fan{i+1}' for i in range(fan_count)]
-        self._led_names = [f'led{i+1}' for i in range(led_channels)]
+        if led_channels == 1:
+            self._led_names = ['led']
+        else:
+            self._led_names = [f'led{i+1}' for i in range(led_channels)]
         self._temp_probs = temp_probs
         self._fan_count = fan_count
 
@@ -148,20 +162,12 @@ class CommanderPro(UsbHidDriver):
             self._data = RuntimeStorage(key_prefixes=[ids, loc])
         return ret
 
-    def initialize(self, **kwargs):
-        """Initialize the device and get the fan modes.
-
-        The device should be initialized every time it is powered on, including when
-        the system resumes from suspending to memory.
-
-        Returns a list of `(property, value, unit)` tuples.
-        """
-
+    def _initialize_directly(self, fan_modes: dict, **kwargs):
         res = self._send_command(_CMD_GET_FIRMWARE)
         fw_version = (res[1], res[2], res[3])
 
         res = self._send_command(_CMD_GET_BOOTLOADER)
-        bootloader_version = (res[1], res[2])               # is it possible for there to be a third value?
+        bootloader_version = (res[1], res[2])  # is it possible for there to be a third value?
 
         status = [
             ('Firmware version', '{}.{}.{}'.format(*fw_version), ''),
@@ -173,76 +179,169 @@ class CommanderPro(UsbHidDriver):
             temp_connected = res[1:5]
             self._data.store('temp_sensors_connected', temp_connected)
             status += [
-                ('Temp sensor 1', 'Connected' if temp_connected[0] else 'Not Connected', ''),
-                ('Temp sensor 2', 'Connected' if temp_connected[1] else 'Not Connected', ''),
-                ('Temp sensor 3', 'Connected' if temp_connected[2] else 'Not Connected', ''),
-                ('Temp sensor 4', 'Connected' if temp_connected[3] else 'Not Connected', ''),
+                (f'Temperature probe {i + 1}', bool(temp_connected[i]), '')
+                for i in range(4)
             ]
 
         if self._fan_count > 0:
-            # get the information about how the fans are connected, probably want to save this for later
+            for i, value in fan_modes.items():
+                fan_num = int(i) - 1
+                if value not in ['dc', 'pwm', 'off']:
+                    raise ValueError(f"invalid fan mode: '{value}'")
+
+                self._send_command(_CMD_SET_FAN_MODE, [0x02, fan_num, _FAN_MODES.get(value)])
+
             res = self._send_command(_CMD_GET_FAN_MODES)
             fanModes = res[1:self._fan_count+1]
             self._data.store('fan_modes', fanModes)
             status += [
-                ('Fan 1 Mode', _get_fan_mode_description(fanModes[0]), ''),
-                ('Fan 2 Mode', _get_fan_mode_description(fanModes[1]), ''),
-                ('Fan 3 Mode', _get_fan_mode_description(fanModes[2]), ''),
-                ('Fan 4 Mode', _get_fan_mode_description(fanModes[3]), ''),
-                ('Fan 5 Mode', _get_fan_mode_description(fanModes[4]), ''),
-                ('Fan 6 Mode', _get_fan_mode_description(fanModes[5]), ''),
+                (f'Fan {i + 1} control mode', _fan_mode_desc(fanModes[i]), '')
+                for i in range(6)
             ]
 
         return status
 
-    def get_status(self, **kwargs):
+    def _get_static_info_from_hwmon(self):
+        # firmware and bootloader versions are not available through hwmon, but
+        # we don't want to race with the kernel driver
+        _LOGGER.warning('some attributes cannot be read from %s kernel driver', self._hwmon.driver)
+
+        status = []
+
+        if self._temp_probs > 0:
+            # use ints to mimic how we normally handle the raw data
+            sensors = [int(self._hwmon.has_attribute(f'temp{n}_input')) for n in range(1, 5)]
+            _LOGGER.debug('%r', sensors)
+            self._data.store('temp_sensors_connected', sensors)
+
+            for n, connected in zip(range(1, 5), sensors):
+                status.append((f'Temperature probe {n}', bool(connected), ''))
+
+        if self._fan_count > 0:
+            def hwmon_fan_mode(hwmon, n):
+                attr = f'fan{n}_label'
+                if not hwmon.has_attribute(attr):
+                    return _FAN_MODE_DISCONNECTED
+
+                label = hwmon.get_string(attr)
+                if label.endswith('4pin'):
+                    return _FAN_MODE_PWM
+                elif label.endswith('3pin'):
+                    return _FAN_MODE_DC
+                else:
+                    assert label.endswith('other')
+                    _LOGGER.warning('hwmon reported the fan mode as other')
+                    return None
+
+            fans = [hwmon_fan_mode(self._hwmon, n) for n in range(1, 7)]
+            _LOGGER.debug('%r', fans)
+            self._data.store('fan_modes', fans)
+
+            for n, mode in zip(range(1, 7), fans):
+                status.append((f'Fan {n} control mode', _fan_mode_desc(mode), ''))
+
+        return status
+
+    def initialize(self, direct_access=False, fan_modes={}, **kwargs):
+        """Initialize the device and the driver.
+
+        This method should be called every time the systems boots, resumes from
+        a suspended state, or if the device has just been (re)connected.  In
+        those scenarios, no other method, except `connect()` or `disconnect()`,
+        should be called until the device and driver has been (re-)initialized.
+
+        Returns None or a list of `(property, value, unit)` tuples, similarly
+        to `get_status()`.
+        """
+
+        if self._hwmon and not direct_access:
+            _LOGGER.info('bound to %s kernel driver, assuming it is already initialized',
+                         self._hwmon.driver)
+
+            if fan_modes:
+                # say "with" instead of "by" because eventually the driver may start supporting fan
+                # modes, and we're not actually checking for it at runtime
+                _LOGGER.warning('fan mode options not supported with %s kernel driver, ignoring',
+                                self._hwmon.driver)
+            return self._get_static_info_from_hwmon()
+        else:
+            if self._hwmon:
+                _LOGGER.warning('forcing re-initialization despite %s kernel driver',
+                                self._hwmon.driver)
+            return self._initialize_directly(fan_modes)
+
+    def _get_status_directly(self):
+        temp_probes = self._data.load('temp_sensors_connected', default=[0]*self._temp_probs)
+        fan_modes = self._data.load('fan_modes', default=[0]*self._fan_count)
+
+        status = []
+
+        # get the temperature sensor values
+        for i, probe_enabled in enumerate(temp_probes):
+            if probe_enabled:
+                temp = self._get_temp(i)
+                status.append((f'Temperature {i + 1}', temp, '°C'))
+
+        # get fan RPMs of connected fans
+        for i, fan_mode in enumerate(fan_modes):
+            if fan_mode == _FAN_MODE_DC or fan_mode == _FAN_MODE_PWM:
+                speed = self._get_fan_rpm(i)
+                status.append((f'Fan {i + 1} speed', speed, 'rpm'))
+
+        # get the real power supply voltages
+        for i, rail in enumerate(["+12V", "+5V", "+3.3V"]):
+            raw = self._send_command(_CMD_GET_VOLTS, [i])
+            voltage = u16be_from(raw, offset=1) / 1000
+            status.append((f'{rail} rail', voltage, 'V'))
+
+        return status
+
+    def _get_status_from_hwmon(self):
+        temp_probes = self._data.load('temp_sensors_connected', default=[0]*self._temp_probs)
+        fan_modes = self._data.load('fan_modes', default=[0]*self._fan_count)
+
+        status = []
+
+        # get the temperature sensor values
+        for i, probe_enabled in enumerate(temp_probes):
+            if probe_enabled:
+                n = i + 1
+                temp = self._hwmon.read_int(f'temp{n}_input') * 1e-3
+                status.append((f'Temperature {n}', temp, '°C'))
+
+        # get fan RPMs of connected fans
+        for i, fan_mode in enumerate(fan_modes):
+            if fan_mode == _FAN_MODE_DC or fan_mode == _FAN_MODE_PWM:
+                n = i + 1
+                speed = self._hwmon.read_int(f'fan{n}_input')
+                status.append((f'Fan {n} speed', speed, 'rpm'))
+
+        # get the real power supply voltages
+        for i, rail in enumerate(["+12V", "+5V", "+3.3V"]):
+            voltage = self._hwmon.read_int(f'in{i}_input') * 1e-3
+            status.append((f'{rail} rail', voltage, 'V'))
+
+        return status
+
+    def get_status(self, direct_access=False, **kwargs):
         """Get a status report.
 
         Returns a list of `(property, value, unit)` tuples.
         """
 
-        if self.device.product_id != 0x0c10:
-            _LOGGER.debug('only the commander pro supports this')
+        if self._fan_count == 0 or self._temp_probs == 0:
+            _LOGGER.debug('only Commander Pro and Obsidian 1000D report status')
             return []
 
-        connected_temp_sensors = self._data.load('temp_sensors_connected', default=[0]*self._temp_probs)
-        fan_modes = self._data.load('fan_modes', default=[0]*self._fan_count)
+        if self._hwmon and not direct_access:
+            _LOGGER.info('bound to %s kernel driver, reading status from hwmon', self._hwmon.driver)
+            return self._get_status_from_hwmon()
 
-        # get the temperature sensor values
-        temp = [0]*self._temp_probs
-        for num, enabled in enumerate(connected_temp_sensors):
-            if enabled:
-                temp[num] = self._get_temp(num)
+        if self._hwmon:
+            _LOGGER.warning('directly reading the status despite %s kernel driver',
+                            self._hwmon.driver)
 
-        # get the real power supply voltages
-        res = self._send_command(_CMD_GET_VOLTS, [0])
-        volt_12 = u16be_from(res, offset=1) / 1000
-
-        res = self._send_command(_CMD_GET_VOLTS, [1])
-        volt_5 = u16be_from(res, offset=1) / 1000
-
-        res = self._send_command(_CMD_GET_VOLTS, [2])
-        volt_3 = u16be_from(res, offset=1) / 1000
-
-        # get fan RPMs of connected fans
-        fanspeeds = [0]*self._fan_count
-        for fan_num, mode in enumerate(fan_modes):
-            if mode == _FAN_MODE_DC or mode == _FAN_MODE_PWM:
-                fanspeeds[fan_num] = self._get_fan_rpm(fan_num)
-
-        status = [
-            ('12 volt rail', volt_12, 'V'),
-            ('5 volt rail', volt_5, 'V'),
-            ('3.3 volt rail', volt_3, 'V'),
-        ]
-
-        for temp_num in range(self._temp_probs):
-            status += [(f'Temp sensor {temp_num + 1}', temp[temp_num], '°C')]
-
-        for fan_num in range(self._fan_count):
-            status += [(f'Fan {fan_num + 1} speed', fanspeeds[fan_num], 'rpm')]
-
-        return status
+        return self._get_status_directly()
 
     def _get_temp(self, sensor_num):
         """This will get the temperature in degrees celsius for the specified temp sensor.
@@ -282,25 +381,21 @@ class CommanderPro(UsbHidDriver):
         """This will get a list of all the fan channels that the command should be sent to
         It will look up the name of the fan channel given and return a list of the real fan number
         """
-        channel = channel.lower()
-        if channel == 'sync':
-            return [i for i in range(len(self._fan_names))]
-        elif channel in self._fan_names:
+        if channel == 'sync' or len(self._fan_names) == 1:
+            return list(range(len(self._fan_names)))
+        if channel in self._fan_names:
             return [self._fan_names.index(channel)]
-        else:
-            raise ValueError(f'unknown channel, should be one of: {_quoted("sync", *self._fan_names)}')
+        raise ValueError(f'unknown channel, should be one of: {_quoted("sync", *self._fan_names)}')
 
     def _get_hw_led_channels(self, channel):
         """This will get a list of all the led channels that the command should be sent to
         It will look up the name of the led channel given and return a list of the real led device number
         """
-        channel = channel.lower()
-        if channel == 'led':
-            return [i for i in range(len(self._led_names))]
-        elif channel in self._led_names:
+        if channel == 'sync' or len(self._led_names) == 1:
+            return list(range(len(self._led_names)))
+        if channel in self._led_names:
             return [self._led_names.index(channel)]
-        else:
-            raise ValueError(f'unknown channel, should be one of: {_quoted("led", *self._led_names)}')
+        raise ValueError(f'unknown channel, should be one of: {_quoted("sync", *self._led_names)}')
 
     def set_fixed_speed(self, channel, duty, **kwargs):
         """Set fan or fans to a fixed speed duty.
@@ -308,11 +403,6 @@ class CommanderPro(UsbHidDriver):
         Valid channel values are 'fanN', where N >= 1 is the fan number, and
         'fan', to simultaneously configure all fans.  Unconfigured fan channels
         may default to 100% duty.
-
-        Different commands for sending fixed percent (0x23) and fixed rpm (0x24)
-        Probably want to use fixed percent for this untill the rpm flag is enabled.
-        Can only send one fan command at a time, if fan mode is unset will need to send 6?
-        messages (or 1 per enabled fan)
         """
 
         if self._fan_count == 0:
@@ -328,16 +418,19 @@ class CommanderPro(UsbHidDriver):
                 self._send_command(_CMD_SET_FAN_DUTY, [fan, duty])
 
     def set_speed_profile(self, channel, profile, temperature_sensor=1, **kwargs):
-        """Set fan or fans to follow a speed duty profile.
+        """Set fan or fans to follow a speed profile _in rpm._
+
+        Unlike corresponding methods in other drivers, this function operates
+        with angular speeds in rpm, not duty values in percentage.
 
         Valid channel values are 'fanN', where N >= 1 is the fan number, and
         'fan', to simultaneously configure all fans.  Unconfigured fan channels
         may default to 100% duty.
 
-        Up to six (temperature, duty) pairs can be supplied in `profile`,
-        with temperatures in Celsius and duty values in percentage.  The last
-        point should set the fan to 100% duty cycle, or be omitted; in the
-        latter case the fan will be set to max out at 60°C.
+        Up to six (temperature, speed) pairs can be supplied in `profile`,
+        with temperatures in Celsius and speeds in rpm.  The last point should
+        set the fan to 100% duty cycle, or be omitted; in the latter case the
+        fan will be heuristically set to max out at 60°C.
         """
 
         # send fan num, temp sensor, check to make sure it is actually enabled, and do not let the user send external sensor
@@ -382,29 +475,10 @@ class CommanderPro(UsbHidDriver):
                 buf[0] = fan
                 self._send_command(_CMD_SET_FAN_PROFILE, buf)
 
-    def set_color(self, channel, mode_str, colors, direction='forward', speed='medium', start_led=1, maximum_leds=1, **kwargs):
+    def set_color(self, channel, mode, colors, direction='forward',
+                  speed='medium', start_led=1, maximum_leds=_MAX_LEDS, **kwargs):
         """Set the color of each LED.
 
-        In reality the device does not have the concept of different channels
-        or modes, but this driver provides a few for convenience.  Animations
-        still require successive calls to this API.
-
-        The 'led' channel can be used to address individual LEDs, and supports
-        the 'super-fixed', 'fixed' and 'off' modes.
-
-        In 'super-fixed' mode, each color in `colors` is applied to one
-        individual LED, successively.  LEDs for which no color has been
-        specified default to off/solid black.  This is closest to how the
-        device works.
-
-        In 'fixed' mode, all LEDs are set to the first color taken from
-        `colors`.  The `off` mode is equivalent to calling this function with
-        'fixed' and a single solid black color in `colors`.
-
-        The `colors` argument should be an iterable of one or more `[red, blue,
-        green]` triples, where each red/blue/green component is a value in the
-        range 0–255.
-
         The table bellow summarizes the available channels, modes, and their
         associated maximum number of colors for each device family.
 
@@ -421,12 +495,11 @@ class CommanderPro(UsbHidDriver):
         | led      | sequential  |          1 |
         | led      | rainbow     |          0 |
         | led      | rainbow2    |          0 |
-
         """
 
         # a special mode to clear the current led settings.
         # this is usefull if the the user wants to use a led mode for multiple devices
-        if mode_str == 'clear':
+        if mode == 'clear':
             self._data.store('saved_effects', None)
             return
 
@@ -435,44 +508,47 @@ class CommanderPro(UsbHidDriver):
         c = itertools.chain(*((r, g, b) for r, g, b in expanded))
         colors = list(c)
 
-        direction = direction.lower()
-        speed = speed.lower()
-        channel = channel.lower()
-        mode = mode_str.lower()
-
-        # default to channel 1 if channel 2 is not specified.
-        led_channel = 1 if channel == 'led2' else 0
-
-        direction = _LED_DIRECTION_FORWARD if direction == 'forward' else _LED_DIRECTION_BACKWARD
+        direction = map_direction(direction, _LED_DIRECTION_FORWARD, _LED_DIRECTION_BACKWARD)
         speed = _LED_SPEED_SLOW if speed == 'slow' else _LED_SPEED_FAST if speed == 'fast' else _LED_SPEED_MEDIUM
-        start_led = clamp(start_led, 1, 96) - 1
-        num_leds = clamp(maximum_leds, 1, 96-start_led-1)  # there is a current firmware limitation of 96 led's per channel
-        random_colors = 0x00 if mode_str == 'off' or len(colors) != 0 else 0x01
-        mode = _MODES.get(mode, -1)
+        start_led = clamp(start_led, 1, _MAX_LEDS) - 1
+        num_leds = clamp(maximum_leds, 1, _MAX_LEDS - start_led)
+        random_colors = 0x00 if mode == 'off' or len(colors) != 0 else 0x01
+        mode_val = _MODES.get(mode, -1)
 
-        if mode == -1:
-            raise ValueError(f'mode "{mode_str}" is not valid')
+        if mode_val == -1:
+            raise ValueError(f'mode "{mode}" is not valid')
 
-        lighting_effect = {
-                'channel': led_channel,
-                'start_led': start_led,
-                'num_leds': num_leds,
-                'mode': mode,
-                'speed': speed,
-                'direction': direction,
-                'random_colors': random_colors,
-                'colors': colors
-            }
+        # FIXME clears on 'off', while the docs only mention this behavior for 'clear'
+        saved_effects = [] if mode == 'off' else self._data.load('saved_effects', default=[])
 
-        saved_effects = [] if mode_str == 'off' else self._data.load('saved_effects', default=[])
-        saved_effects += [lighting_effect]
+        for led_channel in self._get_hw_led_channels(channel):
 
-        self._data.store('saved_effects', None if mode_str == 'off' else saved_effects)
+            lighting_effect = {
+                    'channel': led_channel,
+                    'start_led': start_led,
+                    'num_leds': num_leds,
+                    'mode': mode_val,
+                    'speed': speed,
+                    'direction': direction,
+                    'random_colors': random_colors,
+                    'colors': colors
+                }
 
-        # start sending the led commands
-        self._send_command(_CMD_RESET_LED_CHANNEL, [led_channel])
-        self._send_command(_CMD_BEGIN_LED_EFFECT, [led_channel])
-        self._send_command(_CMD_SET_LED_CHANNEL_STATE, [led_channel, 0x01])
+            saved_effects += [lighting_effect]
+
+            # check to make sure that too many LED effects are not being sent.
+            # the max seems to be 8 as found here https://github.com/liquidctl/liquidctl/issues/154#issuecomment-762372583
+            if len(saved_effects) > 8:
+                _LOGGER.warning(f'too many lighting effects. Run `liquidctl set {channel} color clear` to reset the effect')
+                return
+
+            # start sending the led commands
+            self._send_command(_CMD_RESET_LED_CHANNEL, [led_channel])
+            self._send_command(_CMD_BEGIN_LED_EFFECT, [led_channel])
+            self._send_command(_CMD_SET_LED_CHANNEL_STATE, [led_channel, 0x01])
+
+        # FIXME clears on 'off', while the docs only mention this behavior for 'clear'
+        self._data.store('saved_effects', None if mode == 'off' else saved_effects)
 
         for effect in saved_effects:
             config = [effect.get('channel'),
@@ -502,3 +578,7 @@ class CommanderPro(UsbHidDriver):
         self.device.write(buf)
         buf = bytes(self.device.read(_RESPONSE_LENGTH))
         return buf
+
+    def set_screen(self, channel, mode, value, **kwargs):
+        """Not supported by this device."""
+        raise NotSupportedByDevice()
