@@ -2,27 +2,28 @@
 
 Supported devices:
 
-- Corsair H100i Platinum
-- Corsair H100i Platinum SE
-- Corsair H115i Platinum
-- Corsair H100i Pro XT
-- Corsair H115i Pro XT
+- Corsair Hydro H100i Platinum
+- Corsair Hydro H100i Platinum SE
+- Corsair Hydro H115i Platinum
+- Corsair Hydro H60i Pro XT
+- Corsair Hydro H100i Pro XT
+- Corsair Hydro H115i Pro XT
+- Corsair Hydro H150i Pro XT
 
-Copyright (C) 2020–2021  Jonas Malaco and contributors
+Copyright Jonas Malaco and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
 """
 
 import itertools
 import logging
 import re
-
 from enum import Enum, unique
 
 from liquidctl.driver.usb import UsbHidDriver
+from liquidctl.error import NotSupportedByDevice
 from liquidctl.keyval import RuntimeStorage
-from liquidctl.pmbus import compute_pec
-from liquidctl.util import clamp, fraction_of_byte, u16le_from, \
-                           normalize_profile, check_unsafe
+from liquidctl.util import RelaxedNamesEnum, clamp, fraction_of_byte, \
+                           u16le_from, normalize_profile, mkCrcFun
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -30,6 +31,7 @@ _REPORT_LENGTH = 64
 _WRITE_PREFIX = 0x3f
 
 _FEATURE_COOLING = 0b000
+_FEATURE_COOLING2 = 0b011
 _CMD_GET_STATUS = 0xff
 _CMD_SET_COOLING = 0x14
 
@@ -65,7 +67,7 @@ class _FanMode(Enum):
 
 
 @unique
-class _PumpMode(Enum):
+class _PumpMode(RelaxedNamesEnum):
     QUIET = 0x0
     BALANCED = 0x1
     EXTREME = 0x2
@@ -87,9 +89,8 @@ def _sequence(storage):
     """
 
     while True:
-        seq = storage.load('sequence', of_type=int, default=0) % 31 + 1
-        storage.store('sequence', seq)
-        yield seq
+        seq = storage.load_store('sequence', lambda x : x % 31 + 1, of_type=int, default=0)
+        yield seq[1]
 
 
 def _prepare_profile(original):
@@ -110,19 +111,52 @@ def _quoted(*names):
 class HydroPlatinum(UsbHidDriver):
     """Corsair Hydro Platinum or Pro XT liquid cooler."""
 
-    SUPPORTED_DEVICES = [
-        (0x1b1c, 0x0c18, None, 'Corsair H100i Platinum (experimental)',
+    _MATCHES = [
+        (0x1b1c, 0x0c18, 'Corsair Hydro H100i Platinum',
             {'fan_count': 2, 'fan_leds': 4}),
-        (0x1b1c, 0x0c19, None, 'Corsair H100i Platinum SE (experimental)',
+        (0x1b1c, 0x0c19, 'Corsair Hydro H100i Platinum SE',
             {'fan_count': 2, 'fan_leds': 16}),
-        (0x1b1c, 0x0c17, None, 'Corsair H115i Platinum (experimental)',
+        (0x1b1c, 0x0c17, 'Corsair Hydro H115i Platinum',
             {'fan_count': 2, 'fan_leds': 4}),
-        (0x1b1c, 0x0c20, None, 'Corsair H100i Pro XT (experimental)',
+        (0x1b1c, 0x0c29, 'Corsair Hydro H60i Pro XT',
             {'fan_count': 2, 'fan_leds': 0}),
-        (0x1b1c, 0x0c21, None, 'Corsair H115i Pro XT (experimental)',
+        (0x1b1c, 0x0c20, 'Corsair Hydro H100i Pro XT',
             {'fan_count': 2, 'fan_leds': 0}),
+        (0x1b1c, 0x0c21, 'Corsair Hydro H115i Pro XT',
+            {'fan_count': 2, 'fan_leds': 0}),
+        (0x1b1c, 0x0c22, 'Corsair Hydro H150i Pro XT',
+            {'fan_count': 3, 'fan_leds': 0}),
     ]
 
+    @classmethod
+    def probe(cls, handle, vendor=None, product=None, release=None,
+              serial=None, match=None, **kwargs):
+        """Probe `handle` and yield corresponding driver instances."""
+
+        # this is modified from BaseUsbDriver.probe to match regardless of
+        # presence of "Hydro", for backward compatibility with 1.5.0 and
+        # previous versions
+
+        for vid, pid, desc, devargs in cls._MATCHES:
+            if (vendor and vendor != vid) or handle.vendor_id != vid:
+                continue
+            if (product and product != pid) or handle.product_id != pid:
+                continue
+            if release and handle.release_number != release:
+                continue
+            if serial and handle.serial_number != serial:
+                continue
+            if match:
+                match = match.lower()
+                descr = desc.lower()
+                if not (match in descr or match in descr.replace('hydro ', '')):
+                    continue
+            consargs = devargs.copy()
+            consargs.update(kwargs)
+            dev = cls(handle, desc, **consargs)
+            _LOGGER.debug('%s identified: %s', cls.__name__, desc)
+            yield dev
+
     def __init__(self, device, description, fan_count, fan_leds, **kwargs):
         super().__init__(device, description, **kwargs)
         self._led_count = 16 + fan_count * fan_leds
@@ -176,7 +210,7 @@ class HydroPlatinum(UsbHidDriver):
         # set the flag so the LED command will need to be set again
         self._data.store('leds_enabled', 0)
 
-        self._data.store('pump_mode', _PumpMode[pump_mode.upper()].value)
+        self._data.store('pump_mode', _PumpMode[pump_mode].value)
         res = self._send_set_cooling()
         fw_version = (res[2] >> 4, res[2] & 0xf, res[3])
         if fw_version < (1, 1, 0):
@@ -191,14 +225,20 @@ class HydroPlatinum(UsbHidDriver):
         """
 
         res = self._send_command(_FEATURE_COOLING, _CMD_GET_STATUS)
-        assert len(self._fan_names) == 2, f'cannot yet parse with {len(self._fan_names)} fans'
-        return [
+
+        info = [
             ('Liquid temperature', res[8] + res[7] / 255, '°C'),
-            ('Fan 1 speed', u16le_from(res, offset=15), 'rpm'),
-            ('Fan 2 speed', u16le_from(res, offset=22), 'rpm'),
-            ('Pump speed', u16le_from(res, offset=29), 'rpm'),
         ]
 
+        channels = [('Fan 1', 14), ('Fan 2', 21), ('Fan 3', 42)][:len(self._fan_names)]
+        channels.append(('Pump', 28))
+
+        for name, base in channels:
+            info.append((f'{name} speed', u16le_from(res, offset=base + 1), 'rpm'))
+            info.append((f'{name} duty', round(res[base] / 255 * 100), '%'))
+
+        return info
+
     def set_fixed_speed(self, channel, duty, **kwargs):
         """Set fan or fans to a fixed speed duty.
 
@@ -262,15 +302,9 @@ class HydroPlatinum(UsbHidDriver):
         | led      | off         | synchronized |        0 |      0 |           0 |
         | led      | fixed       | synchronized |        1 |      1 |           1 |
         | led      | super-fixed | independent  |       24 |     16 |          48 |
-
-        Note: lighting control of Pro XT devices is experimental and requires
-        the `pro_xt_lighting` constant to be supplied in the `unsafe` iterable.
         """
 
-        if 'Pro XT' in self.description:
-            check_unsafe('pro_xt_lighting', error=True, **kwargs)
-
-        channel, mode, colors = channel.lower(), mode.lower(), list(colors)
+        colors = list(colors)
         self._check_color_args(channel, mode, colors)
         if mode == 'off':
             expanded = []
@@ -320,7 +354,6 @@ class HydroPlatinum(UsbHidDriver):
         return len(colors)
 
     def _get_hw_fan_channels(self, channel):
-        channel = channel.lower()
         if channel == 'fan':
             return self._fan_names
         if channel in self._fan_names:
@@ -341,20 +374,21 @@ class HydroPlatinum(UsbHidDriver):
             start_at = 3
         if data:
             buf[start_at: start_at + len(data)] = data
-        buf[-1] = compute_pec(buf[2:-1])
+        buf[-1] = mkCrcFun('crc-8')(buf[2:-1])
         self.device.clear_enqueued_reports()
         self.device.write(buf)
         buf = bytes(self.device.read(_REPORT_LENGTH))
-        if compute_pec(buf[1:]):
+        if mkCrcFun('crc-8')(buf[1:]):
             _LOGGER.warning('response checksum does not match data')
         return buf
 
-    def _send_set_cooling(self):
-        assert len(self._fan_names) <= 2, 'cannot yet fit all fan data'
+    def _generate_cooling_payload(self, fan_names):
+
         data = bytearray(_SET_COOLING_DATA_LENGTH)
         data[0: len(_SET_COOLING_DATA_PREFIX)] = _SET_COOLING_DATA_PREFIX
         data[_PROFILE_LENGTH_OFFSET] = _PROFILE_LENGTH
-        for fan, (imode, iduty, iprofile) in zip(self._fan_names, _FAN_OFFSETS):
+
+        for fan, (imode, iduty, iprofile) in zip(fan_names, _FAN_OFFSETS):
             mode = _FanMode(self._data.load(f'{fan}_mode', of_type=int))
             if mode is _FanMode.FIXED_DUTY:
                 stored = self._data.load(f'{fan}_duty', of_type=int, default=100)
@@ -370,7 +404,23 @@ class HydroPlatinum(UsbHidDriver):
             else:
                 raise ValueError(f'unsupported fan {mode}')
             data[imode] = mode.value
+
+        return data
+
+    def _send_set_cooling(self):
         pump_mode = _PumpMode(self._data.load('pump_mode', of_type=int))
+
+        data = self._generate_cooling_payload(self._fan_names[0:2])
         data[_PUMP_MODE_OFFSET] = pump_mode.value
         _LOGGER.info('setting pump mode to %s', pump_mode.name.lower())
+
+        if len(self._fan_names) == 3:
+            data2 = self._generate_cooling_payload(self._fan_names[2:])
+            data2[_PUMP_MODE_OFFSET] = 0xff
+            self._send_command(_FEATURE_COOLING2, _CMD_SET_COOLING, data=data2)
+
         return self._send_command(_FEATURE_COOLING, _CMD_SET_COOLING, data=data)
+
+    def set_screen(self, channel, mode, value, **kwargs):
+        """Not supported by this device."""
+        raise NotSupportedByDevice()
