@@ -55,6 +55,18 @@ lighting channel supports up to 6 accessories and a total of 40 LEDs.
 A microphone is still present onboard for noise level optimization through CAM
 and AI.
 
+NZXT H1 V2
+----------
+
+The second revision of the NZXT H1 case, labeled H1 V2, ships with a variant of the
+NZXT Smart Device V2 that handles both the internal fans and the AIO pump. Two fan and
+one pump channels are available, where the formers can be controlled via PWM or DC.
+
+The pump speed is not user controllable. The device reports the state, speed and duty
+of each fan channel, as well as the pump speed.
+
+There are no lighting channels available nor an onboard microphone.
+
 RGB & Fan Controller
 --------------------
 
@@ -95,16 +107,18 @@ the devices must be manually initialized by calling `initialize()`.  This will
 cause all connected fans and LED accessories to be detected, and enable status
 updates.  It is recommended to initialize the devices at every boot.
 
-Copyright (C) 2018–2021  Jonas Malaco, CaseySJ and contributors
+Copyright Jonas Malaco, CaseySJ and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
 """
 
 import itertools
 import logging
+import time
 
 from liquidctl.driver.usb import UsbHidDriver
 from liquidctl.error import NotSupportedByDevice
-from liquidctl.util import clamp, Hue2Accessory, HUE2_MAX_ACCESSORIES_IN_CHANNEL
+from liquidctl.util import clamp, map_direction, Hue2Accessory, \
+                           HUE2_MAX_ACCESSORIES_IN_CHANNEL
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -120,17 +134,16 @@ _MIN_DUTY = 0
 _MAX_DUTY = 100
 
 
-class _CommonSmartDeviceDriver(UsbHidDriver):
+class _BaseSmartDevice(UsbHidDriver):
     """Common functions of Smart Device and Grid drivers."""
 
     def __init__(self, device, description, speed_channels, color_channels, **kwargs):
-        """Instantiate a driver with a device handle."""
         super().__init__(device, description)
         self._speed_channels = speed_channels
         self._color_channels = color_channels
 
     def set_color(self, channel, mode, colors, speed='normal', direction='forward', **kwargs):
-        """Set the color mode.
+        """Set the color mode for a specific channel.
 
         Only supported by Smart Device V1/V2 and HUE 2 controllers.
         """
@@ -138,13 +151,8 @@ class _CommonSmartDeviceDriver(UsbHidDriver):
         if not self._color_channels:
             raise NotSupportedByDevice()
 
-        channel = channel.lower()
-        mode = mode.lower()
-        speed = speed.lower()
-        direction = direction.lower()
-
         if 'backwards' in mode:
-            _LOGGER.warning('deprecated mode, move to direction=backwards option')
+            _LOGGER.warning('deprecated mode, move to direction=backward option')
             mode = mode.replace('backwards-', '')
             direction = 'backward'
 
@@ -166,7 +174,8 @@ class _CommonSmartDeviceDriver(UsbHidDriver):
         self._write_colors(cid, mode, colors, sval, direction)
 
     def set_fixed_speed(self, channel, duty, **kwargs):
-        """Set channel to a fixed speed."""
+        """Set channel to a fixed speed duty."""
+
         if channel == 'sync':
             selected_channels = self._speed_channels
         else:
@@ -177,29 +186,35 @@ class _CommonSmartDeviceDriver(UsbHidDriver):
             self._write_fixed_duty(cid, duty)
 
     def set_speed_profile(self, channel, profile, **kwargs):
-        """Not Supported by this device."""
         raise NotSupportedByDevice()
 
     def _write(self, data):
         padding = [0x0]*(self._WRITE_LENGTH - len(data))
         self.device.write(data + padding)
 
-    def _write_colors(self, cid, mode, colors, sval):
+    def _write_colors(self, cid, mode, colors, sval, direction):
         raise NotImplementedError()
 
     def _write_fixed_duty(self, cid, duty):
         raise NotImplementedError()
 
+    def set_screen(self, channel, mode, value, **kwargs):
+        """Not supported by this device."""
+        raise NotSupportedByDevice()
 
-class SmartDevice(_CommonSmartDeviceDriver):
+
+class SmartDevice(_BaseSmartDevice):
     """NZXT Smart Device (V1) or Grid+ V3."""
 
-    SUPPORTED_DEVICES = [
-        (0x1e71, 0x1714, None, 'NZXT Smart Device (V1)', {
+    # support for hwmon: nzxt-grid3, liquidtux
+    # https://github.com/liquidctl/liquidtux/blob/3b80dafead6f/nzxt-grid3.c
+
+    _MATCHES = [
+        (0x1e71, 0x1714, 'NZXT Smart Device (V1)', {
             'speed_channel_count': 3,
             'color_channel_count': 1
         }),
-        (0x1e71, 0x1711, None, 'NZXT Grid+ V3', {
+        (0x1e71, 0x1711, 'NZXT Grid+ V3', {
             'speed_channel_count': 6,
             'color_channel_count': 0
         }),
@@ -248,48 +263,110 @@ class SmartDevice(_CommonSmartDeviceDriver):
                           for i in range(color_channel_count)}
         super().__init__(device, description, speed_channels, color_channels, **kwargs)
 
-    def initialize(self, **kwargs):
-        """Initialize the device.
+    def initialize(self, direct_access=False, **kwargs):
+        """Initialize the device and the driver.
 
-        Detects all connected fans and LED accessories, and allows subsequent
-        calls to get_status.
+        Connected fans and LED accessories are detected.
+
+        This method should be called every time the systems boots, resumes from
+        a suspended state, or if the device has just been (re)connected.  In
+        those scenarios, no other method, except `connect()` or `disconnect()`,
+        should be called until the device and driver has been (re-)initialized.
+
+        Returns None or a list of `(property, value, unit)` tuples, similarly
+        to `get_status()`.
         """
 
-        self._write([0x1, 0x5c])  # initialize/detect connected devices and their type
-        self._write([0x1, 0x5d])  # start reporting
+        if self._hwmon and not direct_access:
+            _LOGGER.info('bound to %s kernel driver, assuming it is already initialized',
+                         self._hwmon.driver)
+        else:
+            if self._hwmon:
+                _LOGGER.warning('forcing re-initialization despite %s kernel driver',
+                                self._hwmon.driver)
+            self._write([0x1, 0x5c])  # initialize/detect connected devices and their type
+            self._write([0x1, 0x5d])  # start reporting
+            self.device.clear_enqueued_reports()
 
-    def get_status(self, **kwargs):
-        """Get a status report.
+        msg = self.device.read(self._READ_LENGTH)
+        fw = tuple(msg[0xb:0xf])
+        _LOGGER.debug('raw firmware version: %r', fw)
 
-        Returns a list of (key, value, unit) tuples.
-        """
+        # NZXT/CAM has simplified how they report the firmware version from its
+        # raw 4-component form
+        fw_human = f'{fw[0]}.{fw[3]}'
+        ret = [('Firmware version', fw_human, '')]
 
-        status = []
+        if self._color_channels:
+            lcount = msg[0x11]
+            ret.append(('LED accessories', lcount, ''))
+            if lcount > 0:
+                ltype, lsize = [('HUE+ Strip', 10), ('Aer RGB', 8)][msg[0x10] >> 3]
+                ret.append(('LED accessory type', ltype, ''))
+                ret.append(('LED count (total)', lcount*lsize, ''))
+
+        return ret
+
+    def _get_status_directly(self):
+        fans = [None] * len(self._speed_channels)
         noise = []
+
         self.device.clear_enqueued_reports()
-        for i, _ in enumerate(self._speed_channels):
+        for i, _ in enumerate(fans):
             msg = self.device.read(self._READ_LENGTH)
             num = (msg[15] >> 4) + 1
             state = msg[15] & 0x3
-            status.append((f'Fan {num}', ['—', 'DC', 'PWM'][state], ''))
+
+            fans[num - 1] = [
+                (f'Fan {num} speed', msg[3] << 8 | msg[4], 'rpm'),
+                (f'Fan {num} voltage', msg[7] + msg[8]/100, 'V'),
+                (f'Fan {num} current', msg[9] + msg[10]/100, 'A'),
+                (f'Fan {num} control mode', [None, 'DC', 'PWM'][state], ''),
+            ]
             noise.append(msg[1])
-            if state:
-                status.append((f'Fan {num} speed', msg[3] << 8 | msg[4], 'rpm'))
-                status.append((f'Fan {num} voltage', msg[7] + msg[8]/100, 'V'))
-                status.append((f'Fan {num} current', msg[10]/100, 'A'))
-            if i != 0:
-                continue
-            fw = '{}.{}.{}'.format(msg[0xb], msg[0xc] << 8 | msg[0xd], msg[0xe])
-            status.append(('Firmware version', fw, ''))
-            if self._color_channels:
-                lcount = msg[0x11]
-                status.append(('LED accessories', lcount, ''))
-                if lcount > 0:
-                    ltype, lsize = [('HUE+ Strip', 10), ('Aer RGB', 8)][msg[0x10] >> 3]
-                    status.append(('LED accessory type', ltype, ''))
-                    status.append(('LED count (total)', lcount*lsize, ''))
-        status.append(('Noise level', round(sum(noise)/len(noise)), 'dB'))
-        return sorted(status)
+
+        # flatten fan data while checking for holes
+        ret = []
+        for i, fan in enumerate(fans):
+            if fan:
+                ret = ret + fan
+            else:
+                _LOGGER.warning('missing data fan for %d', i + 1)
+
+        ret.append(('Noise level', round(sum(noise)/len(noise)), 'dB'))
+
+        return ret
+
+    def _get_status_from_hwmon(self):
+        ret = []
+        mode = ['DC', 'PWM']  # slightly simplified, but the device treats undetected == PWM
+
+        for i in range(len(self._speed_channels)):
+            n = i + 1
+            ret.append((f'Fan {n} speed', self._hwmon.read_int(f'fan{n}_input'), 'rpm')),
+            ret.append((f'Fan {n} voltage', self._hwmon.read_int(f'in{i}_input') * 1e-3, 'V')),
+            ret.append((f'Fan {n} current', self._hwmon.read_int(f'curr{n}_input') * 1e-3, 'A')),
+            ret.append((f'Fan {n} control mode', mode[self._hwmon.read_int(f'pwm{n}_mode')], '')),
+
+        # noise level is not available through hwmon, but also not very accurate or useful
+
+        return ret
+
+    def get_status(self, direct_access=False, **kwargs):
+        """Get a status report.
+
+        Returns a list of `(property, value, unit)` tuples.
+        """
+
+        if self._hwmon and not direct_access:
+            _LOGGER.info('bound to %s kernel driver, reading status from hwmon', self._hwmon.driver)
+            return self._get_status_from_hwmon()
+
+        if self._hwmon:
+            _LOGGER.warning('directly reading the status despite %s kernel driver',
+                            self._hwmon.driver)
+
+        return self._get_status_directly()
 
     def _write_colors(self, cid, mode, colors, sval, direction='forward'):
         mval, mod3, mod4, _, _ = self._COLOR_MODES[mode]
@@ -297,8 +374,7 @@ class SmartDevice(_CommonSmartDeviceDriver):
         # one step, where it is specified to all leds and the device handles the animation;
         # but in super mode there is a single step and each color directly controls a led
 
-        if direction == 'backward':
-            mod3 += 0x10
+        mod3 += map_direction(direction, 0, 0x10)
 
         if 'super' in mode:
             steps = [list(itertools.chain(*colors))]
@@ -314,26 +390,52 @@ class SmartDevice(_CommonSmartDeviceDriver):
         self._write([0x2, 0x4d, cid, 0, duty])
 
 
-class SmartDevice2(_CommonSmartDeviceDriver):
+class SmartDevice2(_BaseSmartDevice):
     """NZXT HUE 2 lighting and, optionally, fan controller."""
 
-    SUPPORTED_DEVICES = [
-        (0x1e71, 0x2006, None, 'NZXT Smart Device V2', {
+    # support for hwmon: nzxt-smart2, Linux 5.17
+
+    _MATCHES = [
+        (0x1e71, 0x2006, 'NZXT Smart Device V2', {
             'speed_channel_count': 3,
             'color_channel_count': 2
         }),
-        (0x1e71, 0x2001, None, 'NZXT HUE 2', {
+        (0x1e71, 0x200d, 'NZXT Smart Device V2', {
+            'speed_channel_count': 3,
+            'color_channel_count': 2
+        }),
+        (0x1e71, 0x200f, 'NZXT Smart Device V2', {
+            'speed_channel_count': 3,
+            'color_channel_count': 2
+        }),
+        (0x1e71, 0x2001, 'NZXT HUE 2', {
             'speed_channel_count': 0,
             'color_channel_count': 4
         }),
-        (0x1e71, 0x2002, None, 'NZXT HUE 2 Ambient', {
+        (0x1e71, 0x2002, 'NZXT HUE 2 Ambient', {
             'speed_channel_count': 0,
             'color_channel_count': 2
         }),
-        (0x1e71, 0x2009, None, 'NZXT RGB & Fan Controller', {
+        (0x1e71, 0x2009, 'NZXT RGB & Fan Controller', {
             'speed_channel_count': 3,
             'color_channel_count': 2
         }),
+        (0x1e71, 0x200e, 'NZXT RGB & Fan Controller', {
+            'speed_channel_count': 3,
+            'color_channel_count': 2
+        }),
+        (0x1e71, 0x2010, 'NZXT RGB & Fan Controller', {
+            'speed_channel_count': 3,
+            'color_channel_count': 2
+        }),
+        (0x1e71, 0x2011, 'NZXT RGB & Fan Controller (3+6 channels) (experimental)', {
+            'speed_channel_count': 3,
+            'color_channel_count': 0
+        }),
+        (0x1e71, 0x2019, 'NZXT RGB & Fan Controller (3+6 channels) (experimental)', {
+            'speed_channel_count': 3,
+            'color_channel_count': 0
+        }),
     ]
 
     _MAX_READ_ATTEMPTS = 12
@@ -392,31 +494,47 @@ class SmartDevice2(_CommonSmartDeviceDriver):
                           for i in range(speed_channel_count)}
         color_channels = {f'led{i + 1}': (1 << i)
                           for i in range(color_channel_count)}
-        color_channels['sync'] = (1 << color_channel_count) - 1
+        if color_channels:
+            color_channels['sync'] = (1 << color_channel_count) - 1
         super().__init__(device, description, speed_channels, color_channels, **kwargs)
 
-    def initialize(self, **kwargs):
-        """Initialize the device.
+    def initialize(self, direct_access=False, **kwargs):
+        """Initialize the device and the driver.
 
-        Detects and reports all connected fans and LED accessories, and allows
-        subsequent calls to get_status.
+        Connected fans and LED accessories are detected.
 
-        Returns a list of (key, value, unit) tuples.
+        This method should be called every time the systems boots, resumes from
+        a suspended state, or if the device has just been (re)connected.  In
+        those scenarios, no other method, except `connect()` or `disconnect()`,
+        should be called until the device and driver has been (re-)initialized.
+
+        Returns None or a list of `(property, value, unit)` tuples, similarly
+        to `get_status()`.
         """
 
         self.device.clear_enqueued_reports()
-        # initialize
-        update_interval = (lambda secs: 1 + round((secs - .5) / .25))(.5)  # see issue #128
-        self._write([0x60, 0x02, 0x01, 0xe8, update_interval, 0x01, 0xe8, update_interval])
-        self._write([0x60, 0x03])
+
+        # if fan controller, initialize fan reporting (#331)
+        if self._speed_channels:
+            if self._hwmon and not direct_access:
+                _LOGGER.info('bound to %s kernel driver, assuming it is already initialized',
+                             self._hwmon.driver)
+            else:
+                if self._hwmon:
+                    _LOGGER.warning('forcing re-initialization despite %s kernel driver',
+                                    self._hwmon.driver)
+                update_interval = (lambda secs: 1 + round((secs - .5) / .25))(.5)  # see issue #128
+                self._write([0x60, 0x02, 0x01, 0xe8, update_interval, 0x01, 0xe8, update_interval])
+                self._write([0x60, 0x03])
+
         # request static infos
         self._write([0x10, 0x01])  # firmware info
         self._write([0x20, 0x03])  # lighting info
-        status = []
+        ret = []
 
         def parse_firm_info(msg):
             fw = f'{msg[0x11]}.{msg[0x12]}.{msg[0x13]}'
-            status.append(('Firmware version', fw, ''))
+            ret.append(('Firmware version', fw, ''))
 
         def parse_led_info(msg):
             channel_count = msg[14]
@@ -426,36 +544,68 @@ class SmartDevice2(_CommonSmartDeviceDriver):
                     accessory_id = msg[offset + c * HUE2_MAX_ACCESSORIES_IN_CHANNEL + a]
                     if accessory_id == 0:
                         break
-                    status.append((f'LED {c + 1} accessory {a + 1}',
+                    ret.append((f'LED {c + 1} accessory {a + 1}',
                                    Hue2Accessory(accessory_id), ''))
 
-        self._read_until({b'\x11\x01': parse_firm_info, b'\x21\x03': parse_led_info})
-        return sorted(status)
+        parsers = {b'\x11\x01': parse_firm_info}
+        if self._color_channels:
+            parsers[b'\x21\x03'] = parse_led_info
+        self._read_until(parsers)
+        return sorted(ret)
 
-    def get_status(self, **kwargs):
+    def _get_status_directly(self):
+        ret = []
+
+        def parse_fan_info(msg):
+            mode_offset = 16
+            rpm_offset = 24
+            duty_offset = 40
+            noise_offset = 56
+            raw_modes = [None, 'DC', 'PWM']
+
+            for i, _ in enumerate(self._speed_channels):
+                mode = raw_modes[msg[mode_offset + i]]
+                ret.append((f'Fan {i + 1} speed', msg[rpm_offset + 1] << 8 | msg[rpm_offset], 'rpm'))
+                ret.append((f'Fan {i + 1} duty', msg[duty_offset + i], '%'))
+                ret.append((f'Fan {i + 1} control mode', mode, ''))
+                rpm_offset += 2
+            ret.append(('Noise level', msg[noise_offset], 'dB'))
+
+        self.device.clear_enqueued_reports()
+        self._read_until({b'\x67\x02': parse_fan_info})
+        return sorted(ret)
+
+    def _get_status_from_hwmon(self):
+        ret = []
+        modes = ['DC', 'PWM']
+
+        for n in range(1, len(self._speed_channels) + 1):
+            ret.append((f'Fan {n} speed', self._hwmon.read_int(f'fan{n}_input'), 'rpm')),
+            ret.append((f'Fan {n} duty', self._hwmon.read_int(f'pwm{n}') * 100. / 255, '%')),
+            ret.append((f'Fan {n} control mode', modes[self._hwmon.read_int(f'pwm{n}_mode')], '')),
+
+        # noise level is not available through hwmon, but also not very accurate or useful
+
+        return sorted(ret)
+
+    def get_status(self, direct_access=False, **kwargs):
         """Get a status report.
 
-        Returns a list of (key, value, unit) tuples.
+        Returns a list of `(property, value, unit)` tuples.
         """
 
         if not self._speed_channels:
             return []
-        status = []
 
-        def parse_fan_info(msg):
-            rpm_offset = 24
-            duty_offset = 40
-            noise_offset = 56
-            for i, _ in enumerate(self._speed_channels):
-                if ((msg[rpm_offset] != 0x0) and (msg[rpm_offset + 1] != 0x0)):
-                    status.append((f'Fan {i + 1} speed', msg[rpm_offset + 1] << 8 | msg[rpm_offset], 'rpm'))
-                    status.append((f'Fan {i + 1} duty', msg[duty_offset + i], '%'))
-                rpm_offset += 2
-            status.append(('Noise level', msg[noise_offset], 'dB'))
+        if self._hwmon and not direct_access:
+            _LOGGER.info('bound to %s kernel driver, reading status from hwmon', self._hwmon.driver)
+            return self._get_status_from_hwmon()
 
-        self.device.clear_enqueued_reports()
-        self._read_until({b'\x67\x02': parse_fan_info})
-        return sorted(status)
+        if self._hwmon:
+            _LOGGER.warning('directly reading the status despite %s kernel driver',
+                            self._hwmon.driver)
+
+        return self._get_status_directly()
 
     def _read_until(self, parsers):
         for _ in range(self._MAX_READ_ATTEMPTS):
@@ -469,7 +619,10 @@ class SmartDevice2(_CommonSmartDeviceDriver):
         assert False, f'missing messages (attempts={self._MAX_READ_ATTEMPTS}, missing={len(parsers)})'
 
     def _write_colors(self, cid, mode, colors, sval, direction='forward',):
-        mval, mod3, movingFlag, mincolors, maxcolors = self._COLOR_MODES[mode]
+        mval, mod3, mod4, mincolors, maxcolors = self._COLOR_MODES[mode]
+
+        if not self._color_channels:
+            raise NotSupportedByDevice()
 
         color_count = len(colors)
         if maxcolors == 40:
@@ -496,8 +649,8 @@ class SmartDevice2(_CommonSmartDeviceDriver):
                     self._write(msg + color_lists[i % 4])
                 self._write([0x22, 0x03, cid, 0x08])   # this actually enables wings mode
         else:
-            byte7 = movingFlag  # sets 'moving' flag for moving alternating modes
-            byte8 = direction == 'backward'  # sets 'backwards' flag
+            byte7 = (mod4 & 0x10) >> 4  # sets 'moving' flag for moving alternating modes
+            byte8 = map_direction(direction, 0, 1)  # sets 'backward' flag
             byte9 = mod3 if mval == 0x03 else color_count  # specifies 'marquee' LED size
             byte10 = mod3 if mval == 0x05 else 0x00  # specifies LED size for 'alternating' modes
             header = [0x28, 0x03, cid, 0x00, mval, sval, byte7, byte8, byte9, byte10]
@@ -509,7 +662,40 @@ class SmartDevice2(_CommonSmartDeviceDriver):
         self._write(msg)
 
 
-# backwards compatibility
+class H1V2(SmartDevice2):
+    _MATCHES = [
+        (0x1e71, 0x2015, 'NZXT H1 V2 (experimental)', {
+            'speed_channel_count': 2,
+            'color_channel_count': 0
+        }),
+    ]
+
+    def get_status(self, direct_access=False, **kwargs):
+        ret = []
+
+        def parse_fan_info(msg):
+            mode_offset = 21
+            rpm_offset = 24
+            duty_offset = 25
+            pump_offset = 18
+            raw_modes = [None, 'DC', 'PWM']
+
+            for i, _ in enumerate(self._speed_channels):
+                mode = raw_modes[msg[mode_offset + i]]
+                ret.append((f'Fan {i + 1} speed', msg[rpm_offset] << 8 | msg[rpm_offset - 1], 'rpm'))
+                ret.append((f'Fan {i + 1} duty', msg[duty_offset], '%'))
+                ret.append((f'Fan {i + 1} control mode', mode, ''))
+                rpm_offset += 5
+                duty_offset += 5
+            ret.append(('Pump speed', msg[pump_offset] << 8 | msg[pump_offset - 1], 'rpm'))
+
+        # parse fans and pump status
+        self.device.clear_enqueued_reports()
+        self._read_until({b'\x75\x02': parse_fan_info})
+        return sorted(ret)
+
+
+# backward compatibility
 NzxtSmartDeviceDriver = SmartDevice
 SmartDeviceDriver = SmartDevice
 SmartDeviceV2Driver = SmartDevice2
